import re
import os
import json
import time
from tqdm import tqdm
import requests
import altoshift
import pandas as pd
from altoshift.tools import Tools

helper = Tools()

ALTO_SYNC_PATH = '/api/mapping/sync'
ALTO_READ_PATH = '/api/mapping/read'
ALTO_UPSERT_CATEGORY_PATH = '/api/upsert/categories'
ALTO_UPSERT_PATH = '/api/upsert/products'
ALTO_DELETE_PATH = '/api/delete/products'
ALTO_SEARCH_PATH = '/v1/search'

class connect():
    args = {}
    conf = {
        'DEV': False,
        'host': 'https://search.altoshift.com',
        'xEngineId': '5e170781db2bc600759c0a1c',
        'xUserId': '5e1589d81ef4c70058a6b8ad',
        'xToken': 'ca7effded7c27df4274d29e54053b9654df8069cad804bb4a4cceaad',
        'sToken': '5e1589e8db2bc600759c0a1b'
    }

    @classmethod
    def info(self):
        resp = json.dumps(self.conf, indent=2)
        print "=== Configuration ==="
        print resp
        return 
    
    def __init__(self, host='https://api.altoshift.com', xEngineId='', xUserId='',xToken='', sToken='', **kwargs):
        args = kwargs
        if not(args.get('DEV',False)):
            self.conf['host'] = host
            self.conf['xEngineId'] = args.get('engineId',xEngineId)
            self.conf['xUserId'] = args.get('userId',xUserId)
            self.conf['xToken'] = args.get('token',xToken)
            self.conf['sToken'] = args.get('sToken',sToken)
            
        self.headers = {
            'content-type': 'application/json',
            'Accept-Charset': 'UTF-8',
            'x-engine-id': self.conf['xEngineId'],
            'x-user-id': self.conf['xUserId'],
            'x-token': self.conf['xToken']
        }
        self.info()
    

    def requestHandle(self,resp):
        try:
            result =  {'status_code': resp.status_code,
            'response': resp.json() if resp.text else {}}
        except ValueError:
            result =  {'status_code': resp.status_code,
            'response': resp.text}
        return result

    def requestGet(self,**kwargs):
        args = kwargs
        url = args.get('url','')
        headers = args.get('headers','')
        body = args.get('body','')
        resp = requests.get(url,headers=headers)
        result = self.requestHandle(resp)
        return result

    def requestPost(self, **kwargs):
        args = kwargs
        url = args.get('url','')
        headers = args.get('headers','')
        data = args.get('data','')
        resp = requests.post(url,data=json.dumps(data),headers=headers)
        result = self.requestHandle(resp)
        return result

    def mappingSync(self,dt):
        headers = self.headers
        data = {"customFields": dt}
        url = self.conf['host']+ALTO_SYNC_PATH
        return self.requestPost(url=url,headers=headers,data=data)

    def altoReader(self,file):
        datas = helper.altoReader(file)
        return datas
            
    def altoUpsert(self,file):
        datas = self.altoReader(file)
        if(datas):
            resp = self.upsertData(datas)
        return "Finished"

    def upsertItems(self,dt):
        headers = self.headers

        data = {
            "items": dt
        }
        url = self.conf['host']+ALTO_UPSERT_PATH
        resp = self.requestPost(url=url,headers=headers,data=data)
        return resp
    
    def upsertCategories(self,dt):
        headers = self.headers

        data = {
            "categories": dt
        }
        url = self.conf['host']+ALTO_UPSERT_CATEGORY_PATH
        resp = self.requestPost(url=url,headers=headers,data=data)
        return resp

    def deleteItems(self,dt):
        headers = self.headers
        data = {
            "ids": dt
        }
        url = self.conf['host']+ALTO_DELETE_PATH
        return self.requestPost(url=url,headers=headers,data=data)
    
    def mappingRead(self):
        headers = self.headers
        data = {}
        url = self.conf['host']+ALTO_READ_PATH
        result = self.requestGet(url=url,headers=headers)
        return result
        
    def mappingHeader(self):
        resp = self.mappingRead()
        header = []
        for x in resp['response']['mapping']:
            header += [ x['name'] for x in resp['response']['mapping'][x]]
        result = header
        return result

    def getItems(self,query):
        url = result['host']+ALTO_SEARCH_PATH
        url+='?query=%s&token=%s' % (url, result['sToken'])
        return self.requestGet(url=url)
    
    def upsertData(self, dt, generateCategory=False, categoryLabel=None, categorySplit='|', excludeCategory=[], slicePer=500, delay=0.5, excludeAttribute=[]):
        reports = {'count':{
                        'success':0,
                        'fail':0,
                        'error':0
                    },
                    'logs':{
                        'failed':[]
                    }
                  }
        if(generateCategory):
            categories = helper.categoryGenerator(dt, categoryLabel=categoryLabel, categorySplit=categorySplit, excludeCategory=excludeCategory)
            print('=== Upserting Category ===')
            upCategories = list(categories.itervalues())
            upCategories = helper.dataSlicer(upCategories,slicePer)
            for ct in tqdm(upCategories):
                print(self.upsertCategories(ct))
                time.sleep(delay)
            dt = helper.categoryMap(dt, categories, categoryLabel=categoryLabel, categorySplit=categorySplit)
        print('Data attributes : %s' %dt[0].keys())
        dt = [dict((key,value) for key, value in d.iteritems() if key not in excludeAttribute) for d in dt]
        datas = helper.dataSlicer(dt,slicePer)
        print('=== Upserting Data === \n Total data\t: %s \n Slicer (%s)\t: %s (%s) \n Delay upsert\t: %s' % (len(dt),slicePer,len(datas),[len(x) for x in datas],delay))
        for data in tqdm(datas):
            try:
                resp = self.upsertItems(data).get('response',[])
                if(resp.get('success')):
                    resp = resp.get('response',[])
                    reports['count']['success'] += resp.get('inserted',0)
                    reports['count']['fail'] += len(resp.get('failed',[]))
                    reports['logs']['failed'] += (resp.get('failed',[]))
                    time.sleep(delay)
                else:
                    reports['count']['error'] += len(data)
                    reports['logs']['failed'].append(resp)
            except Exception as e:
                resp = e
                reports['logs']['failed'].append(resp)
        print('Upsert Status | Success : %s | Failed : %s | Error : %s' % (reports['count']['success'],reports['count']['fail'],reports['count']['error']))
        return reports

if(__name__ == "__main__"):
    print('Believe in Future.')


import os
import re
import json
import pandas as pd
from pandas.io.json import json_normalize

class Tools(object):
    def __init__(self):
        super(Tools, self).__init__()
        
    def writeToCsv(self,fname,df,sep='\t'):
        df.to_csv(fname, sep=sep, encoding='utf-8')
    
    def writeToJson(self,fname,df):
        df.to_json(fname, orient='records', lines=False)

    def readMysqltoJson(self,query,con):
        df = pd.read_sql(query, con=con)
        result = df.to_json(orient='records', lines=False)
        result = json.loads(result.decode('latin-1'))
        return result

    def readXlsToJson(self,file):
        df = pd.read_excel(file)
        result = df.to_json(orient='records', lines=False)
        result = json.loads(result.decode('latin-1'))
        return result

    def readCsvToJson(self,file,xdelimiter='\t'):
        df = pd.read_csv(file,delimiter=xdelimiter)
        result = df.to_json(orient='records', lines=False)
        result = json.loads(result.decode('latin-1'))
        return result

    def cleanHTMLforJson(self,dt,xfilter=[]):
        df = dt.filter(xfilter) if xfilter else dt
        df = json_normalize(dt)
        for cfilter in xfilter:
            df[cfilter]=re.sub(r'<.+?>','',str(df[cfilter]))
        return df

    def filterJson(self,dt,xfilter=None):
        df = pd.DataFrame(dt)
        df = df.filter(xfilter) if xfilter else df
        result = df.to_json(orient='records', lines=False)
        result = json.loads(result.decode('latin-1'))
        return result

    def dataSlicer(self, source, every=500):
        step = (len(source)/every+1)
        return [source[i::step] for i in range(step)]

    def altoReader(self,file):
        if(os.path.isfile(file) == False):
            print('File not found')
            return False
        
        if(file.lower().endswith('.csv')):
            datas = self.readCsvToJson(file)
        elif(file.lower().endswith('.tsv')):
            datas = self.readCsvToJson(file,'\t')
        elif(file.lower().endswith('.xlsx')):
            datas = self.readXlsToJson(file)
        else:
            print('not supported file')
        return datas

    def altoOptimizer(self,fname,df2):
        df1 = pd.DataFrame()
        if(os.path.isfile(fname)):
            df1 = pd.DataFrame(self.readCsvToJson(fname))
        if not(df1.empty):
            merged = df1.merge(df2, indicator=True, how='outer')
            data = merged[merged['_merge'] == 'right_only']
        else:
            data = df2
        self.writeToCsv(fname,df2)
        result = data.to_json(orient='records', lines=False)
        result = json.loads(result.decode('latin-1'))
        return result
    
    def categoryGenerator(self, dt, categoryLabel=None, categorySplit='|', excludeCategory=[]):
        categoryIdx = 0
        categories = {}
        for row in dt:
            if(categoryLabel == None):
                ctRow = row
            elif(categoryLabel not in row):
                print('Please specify the correct category label | ex : categoryGenerator(dt, categoryLabel="category"')
                break
            else:
                ctRow = row[categoryLabel]
    #           Validate row
            if(type(ctRow) in [str,unicode]):
                ct = [str(c) for c in ctRow.split(categorySplit)]
            elif(type(ctRow) == dict):
                print('%s |Please specify the category label | ex : categoryGenerator(dt, categoryLabel="category")' % (type(row)))
                break
            else:
                ct = []
            for idx,c in enumerate(ct):
                if (c in excludeCategory):
                    continue
                elif(c not in categories):
                    if(idx != 0):
                        parentId = str(categories[ct[idx-1]]['catId'])
                    else:
                        parentId = '0'
                    categories[c] = {'name':c, 'count':1, 'parentCatId':str(parentId), 'catId':str(categoryIdx), 'description':''}
                    categoryIdx+=1
                else:
                    categories[c]['count'] += 1
        return categories

    def categoryMap(self, dt, categories, categoryLabel=None, categorySplit='|'):
        for i,row in enumerate(dt): 
            if(categoryLabel == None):
                ctRow = row
            else:
                ctRow = row[categoryLabel]
            ctRow = ctRow.split(categorySplit)
            ctids = [categories[c]['catId'] for c in ctRow]
            dt[i]['categoryIds'] = ctids
        return dt
    
if(__name__ == "__main__"):
    print('Believe in Future.')
